source_csadm(){
    fsr_csadm_utils="/opt/cyops/adm/utils"
    source $fsr_csadm_utils
}

init(){
    source_csadm
}

validate_input(){
    local context=$1
    local version=$2
    if [ "X$context" = "X" ] || [ "X$version" = "X" ]; then
        echo "$0 upgrade <version-here>"
        exit 1
    fi
}

update_mq_conf_file(){
    local conf_mq="/etc/rabbitmq/rabbitmq.conf"

    if grep -q "listeners.ssl.default" $conf_mq; then
        return
    fi

    local new_config="listeners.tcp.default=5672
listeners.ssl.default=5671
ssl_options.cacertfile=/opt/cyops/configs/rabbitmq/ssl/cyopsca/cacert.pem
ssl_options.certfile=/opt/cyops/configs/rabbitmq/ssl/server/cert.pem
ssl_options.keyfile=/opt/cyops/configs/rabbitmq/ssl/server/key.pem
ssl_options.versions.1=tlsv1.2
ssl_options.honor_cipher_order=true
ssl_options.honor_ecc_order=true
ssl_options.secure_renegotiate=true
ssl_options.verify=verify_peer
ssl_options.fail_if_no_peer_cert=false

ssl_options.ciphers.1=ECDHE-ECDSA-AES256-GCM-SHA384
ssl_options.ciphers.2=ECDHE-RSA-AES256-GCM-SHA384
ssl_options.ciphers.3=ECDH-ECDSA-AES256-GCM-SHA384
ssl_options.ciphers.4=ECDH-RSA-AES256-GCM-SHA384
ssl_options.ciphers.5=DHE-RSA-AES256-GCM-SHA384
ssl_options.ciphers.6=DHE-DSS-AES256-GCM-SHA384
ssl_options.ciphers.7=ECDHE-ECDSA-AES128-GCM-SHA256
ssl_options.ciphers.8=ECDHE-RSA-AES128-GCM-SHA256
ssl_options.ciphers.9=ECDH-ECDSA-AES128-GCM-SHA256
ssl_options.ciphers.10=ECDH-RSA-AES128-GCM-SHA256
ssl_options.ciphers.11=DHE-RSA-AES128-GCM-SHA256
ssl_options.ciphers.12=DHE-DSS-AES128-GCM-SHA256"
    echo "" >> $conf_mq
    echo "$new_config" >> $conf_mq
}

update_mq_env_file_with_new_format_mq_conf_file(){
    local mq_env_file="/etc/rabbitmq/rabbitmq-env.conf"
    local mq_conf_file_env_var="RABBITMQ_CONFIG_FILE"
    local mq_old_config_file_name="rabbitmq.config"
    local mq_ssl_dir="/opt/cyops-rabbitmq/configs/ssl"
    local mq_old_config_file_path="$mq_ssl_dir/$mq_old_config_file_name"
    local mq_new_config_file_name_for_embedded_sme="rabbitmq-embedded-sme.conf"
    local mq_new_config_file_name_for_external_sme="rabbitmq-external-sme.conf"

    if [ ! -f "$mq_env_file" ]; then
        return
    fi

    if ! grep -q $mq_conf_file_env_var $mq_env_file; then
        # key RABBITMQ_CONFIG_FILE not found in file /etc/rabbitmq/rabbitmq-env.conf
        return
    fi

    if ! grep -q "rabbitmq\.config" $mq_env_file; then
        # /etc/rabbitmq/rabbitmq-env.conf is already updated with new format file
        return
    fi

    local old_mngt_port=$(grep "port" $mq_old_config_file_path | awk '{print $2}' | cut -d"}" -f1 | cut -d "[" -f2 | cut -d "]" -f1)

    local old_ssl_port=$(grep "ssl_listeners" $mq_old_config_file_path | awk '{print $2}' | cut -d"}" -f1 | cut -d "[" -f2 | cut -d "]" -f1)

    local mq_new_config_file_name="$mq_new_config_file_name_for_external_sme"

    if is_embedded_sme; then
        mq_new_config_file_name="$mq_new_config_file_name_for_embedded_sme"
    fi

    sed -i "s/rabbitmq\.config/$mq_new_config_file_name/g" $mq_env_file

    local mq_new_config_file_full_path="$mq_ssl_dir/$mq_new_config_file_name"

    sed -i "s/^\s*listeners.ssl.default\s*=.*$/listeners.ssl.default=$old_ssl_port/g" $mq_new_config_file_full_path
    sed -i "s/^\s*management.ssl.port\s*=.*$/management.ssl.port=$old_mngt_port/g" $mq_new_config_file_full_path

    chown root:rabbitmq $mq_env_file $mq_new_config_file_full_path
    chmod 644 $mq_env_file
    chmod 755 $mq_new_config_file_full_path
}

handle_tls_for_mq(){
    local script_create_mq_user="/opt/cyops-rabbitmq/configs/rabbitmq/rabbitmq_configs.sh"
    local mq_tls_uname="fsr-cluster"
    local mq_vhost="fsr-cluster"
    local conf_db="/opt/cyops/configs/database/db_config.yml"
    local mq_old_config_file_name="rabbitmq.config"
    local mq_old_config_file_path="/opt/cyops-rabbitmq/configs/ssl/$mq_old_config_file_name"

    local port_mq_tls=5671

    if is_embedded_sme; then
        port_mq_tls=$(grep "ssl_listeners" $mq_old_config_file_path | awk '{print $2}' | cut -d"}" -f1 | cut -d "[" -f2 | cut -d "]" -f1)
    fi

    #
    # We don't change the TCP port during enterprise configuration or embedded SME configuration.
    # Using it with default value.
    #
    local port_mq_tcp=5672

    local need_to_create_mq_user="False"
    if ! grep -q $mq_tls_uname $conf_db; then
        echo "" >> $conf_db
        echo "mq:" >> $conf_db
        echo "  mq_port: $port_mq_tcp" >> $conf_db
        echo "  mq_ssl_enabled: false" >> $conf_db
        echo "  mq_tls_port: $port_mq_tls" >> $conf_db
        echo "  mq_host: localhost" >> $conf_db
        echo "  mq_user: '$mq_tls_uname'" >> $conf_db
        echo "  mq_password: 'default_password'" >> $conf_db
        echo "  mq_ca_cert: ''" >> $conf_db
        echo "  mq_vhost: '$mq_vhost'" >> $conf_db

        need_to_create_mq_user="True"
    fi

    if [ "X$need_to_create_mq_user" = "XTrue" ]; then
        # TLS user is not already defined. Define it and create it
        sh $script_create_mq_user upgrade mq
    fi

    update_mq_conf_file

    update_mq_env_file_with_new_format_mq_conf_file

    if is_embedded_sme; then
        # Since this is embedded SME configured, the MQ certs are already there.
        return
    fi

    # Generate the MQ certs and no need to share it across cluster as this script
    # gets called during upgrade. We mandate leave-cluster before upgrade.
    #
    csadm mq --generate-certs --no-propagate

    # Above command generate the certs in below directory. Change the perms.

    chown -R root:rabbitmq /opt/cyops/configs/rabbitmq/ssl/
}

is_mq_tls_changes_required(){
    local fsr_version_file="/opt/cyops-ui/cyops_version.json"
    if [ ! -f "$fsr_version_file" ]; then
        # 1 means false
        return 1
    fi
    local curr_version=$(python3 -c "import json;data=json.load(open('$fsr_version_file',));print(data['version'])")
    # We get curr_version like 6.4.3-1234, split on '-' and get version 6.4.3
    curr_version=$(echo "$curr_version" | cut -d'-' -f1)
    local status=$(version_compare "$curr_version" "7.0.0")
    if [ $? -ne 0 ]; then
        # version_compare failed. return 1. 1 means false
        return 1
    fi
    if [ $status -eq 2 ]; then
        # new_version is less than 7.0.0
        # 0 means true
        return 0
    fi
    return 1 
}

handle_upgrade(){
    local new_version=$1
    if is_enterprise_instance; then
        # Checking for enterprise so that embedded sme don't conflict with is_secure_message_instance.
        if is_mq_tls_changes_required; then
            handle_tls_for_mq
        fi
        return
    fi

    if is_secure_message_exchange_instance; then
        # Please note that we are not checking whether current version is less than 7.0.0
        # like we do for enterprise here. This is because below sub-routine are idempotent.
        update_mq_conf_file
        update_mq_env_file_with_new_format_mq_conf_file
    fi
}

handle_default_steps_for_context_upgrade(){
    systemctl restart rabbitmq-server
}

############
# Main/main/MAIN starts here
#########
context=$1
version=$2
# This script usually runs from the specfile. Hence add -x here for debug.
set -x
init
validate_input "$context" "$version"
handle_${context} "$version"
handle_default_steps_for_context_${context}
exit 0
